package eu.dnetlib.validator2.engine;

import eu.dnetlib.validator2.validation.XMLApplicationProfile;
import eu.dnetlib.validator2.validation.guideline.Guideline;
import eu.dnetlib.validator2.validation.guideline.openaire.AbstractOpenAireProfile;
import eu.dnetlib.validator2.validation.guideline.openaire.LiteratureGuidelinesV4Profile;
import groovy.xml.DOMBuilder;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;

import java.io.FileReader;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.OptionalDouble;
import java.util.stream.Collectors;

public class Test_v4 {

    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(Test_v4.class);


    private static final String[] FILES = new String[] {
            "src/test/resources/openaireguidelinesV4/v4_literature_all_invalid_guidelines_record.xml",
//          "src/test/resources/openaireguidelinesV4/v4_literature_all_guidelines_record.xml",
//          "src/test/resources/openaireguidelinesV4/oai_mediarep_org_doc_2534.xml",
//          "src/test/resources/openaireguidelinesV4/01_gv4.xml"
    };

    public static void main(String[] args) {
        // String xmlFile = args[0];
        AbstractOpenAireProfile profile = new LiteratureGuidelinesV4Profile();
        logger.info("Max score: " + profile.maxScore());
        Map<String, Double> scorePerDoc = new LinkedHashMap<>();
        for (String file: FILES) {
            try {
                logger.info("Processing \"" + file + "\"");
                Document doc = DOMBuilder.parse(new FileReader(file), false, true, true);
                XMLApplicationProfile.ValidationResult result = profile.validate(file, doc);
                scorePerDoc.put(file, result.score());
                Map<String, Guideline.Result> results = result.results();
                for (Map.Entry entry : results.entrySet()) {
                    if (entry.getKey().toString().contains("Date")) {
                        logger.info("Warnings: " + results.get(entry.getKey()).warnings().toString());
                        logger.info("Errors: " + results.get(entry.getKey()).errors().toString());
                        logger.info("Result: " + entry.getKey() + " = " + entry.getValue() + "\n");
                    }
                    logger.debug(entry.getKey() + " = " + entry.getValue());
                }
            }
            catch(Exception e) {
                logger.error(e.getMessage());
                e.printStackTrace();
            }
        }
//        Individual scores
        String printout = scorePerDoc.entrySet().stream().
                map(entry -> entry.getValue() + ": " + entry.getKey()).collect(Collectors.joining("\n"));

        //        Average Score
        OptionalDouble optionalFinalScore = scorePerDoc.values().stream().mapToDouble(aDouble -> aDouble).average();
        double finalScore = (optionalFinalScore.isPresent() ? optionalFinalScore.getAsDouble() : -1 );

        logger.info(printout);
        logger.info("\nValidation Score: " + finalScore);
    }

}
